import base64
import json
import mimetypes
import uuid
from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot

USERNAME = ''  # 用户名
REPO = ''  # 仓库名
SAVE_PATH = ''  # 保存路径
TOKEN = ''
USE_JSDELIVR = False  # 是否使用 jsdelivr 加速
BRANCH = ''


def set_config(username: str, repo: str, save_path: str, token: str, use_jsdelivr: bool = True, branch: str = 'master'):
    """
    :param username: 仓库用户名
    :param repo: 仓库名称
    :param save_path: 文件保存路径. eg: content/img
    :param token: 获取地址: https://github.com/settings/tokens <br>
        创建 token 时, 权限选中 repo 的所有
    :param use_jsdelivr: 是否使用 jsdelivr 加速 <br>
        若为 True, 则返回 jsdelivr 图片链接, 否则返回 github 链接 <br>
        默认为 True
    :param branch: 指定分支名称, 分支必须存在. 默认'master'
    :return:
    """
    global USERNAME, REPO, SAVE_PATH, TOKEN, USE_JSDELIVR, BRANCH
    USERNAME = username
    REPO = repo
    SAVE_PATH = save_path
    TOKEN = token
    USE_JSDELIVR = use_jsdelivr
    BRANCH = branch


class Github(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.Github

    # 上传图片, 二进制内容
    def upload(self, content) -> Optional[str]:
        if not USERNAME or not REPO or not SAVE_PATH or not TOKEN:
            return self._set_error('config is miss')
        # 获取文件名称
        file_name = self._random_file_name(content)
        if not file_name:
            return None
        url = f"https://api.github.com/repos/{USERNAME}/{REPO}/contents/{SAVE_PATH}/" + file_name  # 用户名、库名、路径
        data = {
            "message": "upload file",
            "content": base64.b64encode(content).decode(),
            'branch': BRANCH
        }
        headers = {
            "Authorization": "token " + TOKEN
        }
        response = requests.put(url=url, data=json.dumps(data), headers=headers)
        if response.status_code != 201 and response.status_code != 200:
            return self._set_error(f'upload fail. code: {response.status_code}. content: {response.text}')
        # 获取图片链接
        url = response.json().get('content', {}).get('download_url')
        if not url:
            return self._set_error(f'response error. {response.text}')
        # 返回结果
        if USE_JSDELIVR:
            return f"https://cdn.jsdelivr.net/gh/{USERNAME}/{REPO}@{BRANCH}/{SAVE_PATH}/{file_name}"
        else:
            return url



"""
@author hujing
"""
from enum import Enum, auto


class DepotType(Enum):
    CatBox = auto()
    """
    图床
    
    url: https://catbox.moe/
    """
    PngCm = auto()
    """
    图床
    
    url: https://png.cm/
    """
    Riyugo = auto()
    """
    限制: 
        1. 图片3天内有效
    url: https://riyugo.com/
    """
    NiuPic = auto()
    """
    牛客网
    
    url: https://www.niupic.com/
    """
    ImgUrl = auto()
    """
    免费账户有上传限制:
        1. 每日限制上传15张图片
        2. 每月限制上传400张
    图片链接限制: 
        1. 浏览器直接打开图片时, 会重定向到图床网址
    url: https://www.imgurl.org/
    """
    SmMs = auto()
    """
    限制: 
        1. 免费账号限制存储空间(可注册多个账号)
        2. 图片链接空 referer 访问时, 会重定向到图床网址
    url: https://sm.ms/
    """
    Imgbb = auto()
    """
    限制: 
        1. 图片链接空 referer 访问时, 会重定向到图床网址
    url: https://imgbb.com/
    """
    Xywm = auto()
    """
    图床
        1. 注册用户存在限制, 可能登录后查看
            * 限制并发数量及上传数量
    
    url: https://pic.xywm.ltd/
    """
    BoLuo = auto()
    """
    图床, 限制: 
        1. 每天限制上传10张
    url: https://www.boluo.link/
    """
    ImgAx = auto()
    """
    url: https://img.ax/
    """
    HuaBan = auto()
    """
    花瓣网站的图片上传. 图片上传接口在图集上传处发现
    
    限制: 
        1. 需要先登录
        2. 图片做了 referer 判断, 空 referer 可访问
    url: https://huaban.com/
    """
    Ai58 = auto()
    """
    58同城客服, 在线上传图片
    
    url: https://ai.58.com
    """
    Github = auto()
    """
    借用 github 仓库
    
    url: https://github.com
    """



import random
from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot

TOKEN = ''
EMAIL = ''
PASSWORD = ''


def set_config(email: str, password: str):
    """
    :param email:
    :param password:
    :return:
    """
    global EMAIL, PASSWORD
    EMAIL = email
    PASSWORD = password


class Xywm(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.Xywm

    def upload(self, content) -> Optional[str]:
        global TOKEN
        # 若存在用户名和密码, 获取 token
        if not TOKEN and EMAIL and PASSWORD:
            data = {
                'email': EMAIL,
                'password': PASSWORD
            }
            response = requests.post("https://pic.xywm.ltd/api/v1/tokens", data=data)
            if response.status_code == 200:
                TOKEN = response.json().get('data', {}).get('token')

        # 上传图片
        tmp_filename = self._random_file_name(content)
        if not tmp_filename:
            return None
        files = {
            "file": (tmp_filename, content)
        }
        headers = {
            "Accept": "application/json",
        }
        if TOKEN:
            headers['Authorization'] = f'Bearer {TOKEN}'
        response = requests.post("https://pic.xywm.ltd/api/v1/upload", headers=headers, files=files)
        if response.status_code != 200:
            return self._set_error(f'upload fail. code: {response.status_code}')
        data = response.json()
        if not data.get('status'):
            return self._set_error(f'upload fail. {response.text}')
        url = data.get('data', {}).get('links', {}).get('url')
        if not url:
            return self._set_error(f'response error. {response.text}')
        return url



"""
@author hujing
"""
from .depot.sm_ms import set_config as sm_ms_set_config
from .depot.github import set_config as github_set_config
from .depot.img_url import set_config as img_url_set_config
from .depot.hua_ban import set_config as hua_ban_set_config
from .depot.xywm import set_config as xywm_set_config


class DepotConfig:
    SmMs = sm_ms_set_config
    Github = github_set_config
    ImgUrl = img_url_set_config
    HuaBan = hua_ban_set_config
    Xywm = xywm_set_config
